package org.netkernel.packge.endpoint;

import org.netkernel.layer0.boot.ModuleManager;
import org.netkernel.layer0.nkf.INKFRequest;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.NKFException;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.impl.HDSFactory;
import org.netkernel.layer0.util.HDSUtils;
import org.netkernel.module.standard.endpoint.StandardTransportImpl;

public class AppositeContinuationEndpoint extends StandardTransportImpl
{
	public static final String APPOSITE_CONTINUITY_STATE="pds:/apposite/continuityState";
	private ContinuityThread mThread;
	
	public AppositeContinuationEndpoint()
	{	this.declareThreadSafe();
	}

	@Override
	public void onSink(INKFRequestContext context) throws Exception
	{	//Store continuity state in PDS
		IHDSNode state=context.sourcePrimary(IHDSNode.class);
		context.sink(APPOSITE_CONTINUITY_STATE, HDSFactory.toDOM(state));
		String next=(String)state.getFirstValue("/continuity/next");
		mThread=new ContinuityThread(next, 1000);
		mThread.start();
	}

	@Override
	protected void postCommission(INKFRequestContext context) throws Exception
	{	//Restore continuity state from PDS to see if we need to kick the process after we've been reloaded
		try
		{
			IHDSNode state=context.source(APPOSITE_CONTINUITY_STATE, IHDSNode.class);
			if(state!=null)
			{	String next=(String)state.getFirstValue("/continuity/next");
				ContinuityThread t=new ContinuityThread(next, 1000);
				t.start();
			}
		}
		catch(Exception e)
		{	//do nothing state does not exist			
		}
	}

	@Override
	protected void preDecommission(INKFRequestContext context) throws Exception
	{	//We're going down - gently kill the continuity thread - we'll have to resume when we're recommissioned.
		if(mThread!=null && mThread.isAlive())
		{	mInterrupted=true;
			mThread.interrupt();
		}
	}
	
	private boolean mInterrupted=false;

	private class ContinuityThread extends Thread
	{
		String mIdentifier;
		int mDelay;
		
		public ContinuityThread(String aIdentifier, int aDelay)
		{	mIdentifier=aIdentifier;
			mDelay=aDelay;
		}

		@Override
		public void run()
		{	try
			{	INKFRequestContext context=getTransportContext();
				if(mDelay>0)
				{	sleep(mDelay);
					ModuleManager.getSingleton().sync(false);
					if(!mInterrupted)
					{	//We're still alive so we can delete continuity state
						context.delete(APPOSITE_CONTINUITY_STATE);
					}
					else
					{	return;						
					}
				}
				//Kick the process
				INKFRequest req=context.createRequest(mIdentifier);
				context.issueRequest(req);
			}
			catch(InterruptedException e)
			{	//Let it die	
			}
			catch(NKFException e)
			{	e.printStackTrace();
			}
		}
	}
}
